import React, { useState, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { useDataMap } from './useDataMap';
import { useDataNuclear } from './useDataNuclear';
import { Marks } from './Marks';
import { scaleLinear, extent, scaleSqrt, max, scaleOrdinal } from 'd3';
import { ColorLegend } from './ColorLegend';

const width = 960;
const height = 500;
const fadeOpacity = 0.25;

const App = () => {
  const mapData = useDataMap();
  const nuclearData = useDataNuclear();

  const [hoveredValue, setHoveredValue] = useState(null);
  const [clickedValue, setClickedValue] = useState(null);

  if (!mapData || !nuclearData) {
    return <pre>Loading...</pre>;
  }

  const longitude = d => d.longitude;
  const latitude = d => d.latitude;
  const name = d => d.name;

  const sizeValue = d => d.yield_upper;
  const maxRadius = 20;
  const sizeScale = scaleSqrt()
    .domain([0, max(nuclearData, sizeValue)])
    .range([3, maxRadius]);

  const colorValue = d => d.country;
  const colorLegendLabel = 'Country';
  const colorScale = scaleOrdinal()
    .domain(nuclearData.map(colorValue))
    .range([
      '#BD2D28',
      '#E6842A',
      '#E3BA22',
      '#5C8100',
      '#0F8C79',
      '#356B90',
      '#684664'
    ]);

  const filteredNuclearData = nuclearData.filter(
    d => hoveredValue === colorValue(d)
  );

  return (
    <svg width={width} height={height}>
      <Marks
        mapData={mapData}
        nuclearData={filteredNuclearData}
        longitude={longitude}
        latitude={latitude}
        name={name}
        sizeScale={sizeScale}
        sizeValue={sizeValue}
        colorScale={colorScale}
        colorValue={colorValue}
        onClick={setClickedValue}
      />
      <g opacity={hoveredValue ? fadeOpacity : 1}>
        <Marks
          mapData={mapData}
          nuclearData={nuclearData}
          longitude={longitude}
          latitude={latitude}
          name={name}
          sizeScale={sizeScale}
          sizeValue={sizeValue}
          colorScale={colorScale}
          colorValue={colorValue}
          onClick={setClickedValue}
        />
      </g>
      <g transform={`translate(10, 40)`}>
        <text x={35} y={-17} className="axis-label" textAnchor="middle">
          {colorLegendLabel}
        </text>
        <ColorLegend
          colorScale={colorScale}
          tickSpacing={20}
          tickSize={9}
          tickTextOffset={12}
          onHover={setHoveredValue}
          hoveredValue={hoveredValue}
          fadeOpacity={fadeOpacity}
        />
      </g>
      <g transform={`translate(10, 40)`}>
        <text x={1} y={390} className="info">
          {clickedValue ? 'Information:' : ''}
        </text>
        <text x={1} y={402} className="info">
          {clickedValue ? ('Name: ', clickedValue.name) : ''}
        </text>
        <text x={1} y={414} className="info">
          {clickedValue ? ('Year: ', clickedValue.year) : ''}
        </text>
        <text x={1} y={426} className="info">
          {clickedValue ? ('Source: ', clickedValue.source) : ''}
        </text>
        <text x={1} y={438} className="info">
          {clickedValue ? ('Type: ', clickedValue.type) : ''}
        </text>
        <text x={1} y={450} className="info">
          {clickedValue ? ('Magnitude: ', clickedValue.yield_upper) : ''}
        </text>
      </g>
    </svg>
  );
};
const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
